package com.suning.mobile.epa.paymentcode.main

import android.Manifest
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.os.Handler
import android.support.v4.app.ActivityCompat
import android.support.v4.app.Fragment
import android.text.TextUtils
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.suning.epafusion.R
import com.suning.mobile.epa.kits.utils.ActivityLifeCycleUtil
import com.suning.mobile.epa.kits.utils.DeviceInfoUtil
import com.suning.mobile.epa.kits.utils.LogUtils
import com.suning.mobile.epa.kits.utils.ToastUtil
import com.suning.mobile.epa.kits.view.CustomAlertDialog
import com.suning.mobile.epa.kits.view.ProgressViewDialog
import com.suning.mobile.epa.paymentcode.PaymentCodeGlobalInfo
import com.suning.mobile.epa.paymentcode.h5.PaymentPayH5Activity
import com.suning.mobile.epa.paymentcode.network.PaymentNetworkConfig
import com.suning.mobile.epa.paymentcode.open.PaymentCodeOpenUtil
import com.suning.mobile.epa.paymentcode.utils.*
import com.suning.mobile.epa.paymentcode.view.PaymentCodeMethodDialog
import kotlinx.android.synthetic.main.fragment_payment_code_main.*
import java.util.*

class PaymentCodeMainFragment : Fragment() {

    companion object {
        const val pageId = "QyZr"
    }

    private val REQUEST_PHONE = 200

    private var startTime = 0L
    private var getOrderTime = 0L
    private var isPaymentSwitchOn = false

    private var mBarCodeBitmap: Bitmap? = null
    private var mBarCodeWidth = 0
    private var mBarCodeHeight = 0
    private var mQRCodeBitmap: Bitmap? = null
    private var mQRCodeSize = 0

    private var hasUpdateShowTime = false
    private var isSdkPayed = false

    private var mPaymentCodeModel: PaymentCodeModel? = null
    private var mPaymentMethodModel: PaymentMethodModel? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        startTime = System.currentTimeMillis()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        return inflater.inflate(R.layout.fragment_payment_code_main, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView()
        initData()
    }

    override fun onResume() {
        super.onResume()
        // 更新授权码时间等等
        refreshQRCode()
        startTimeService()
    }

    override fun onPause() {
        super.onPause()
        stopTimeService()
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == REQUEST_PHONE) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                sendPaymentCodeAuthIdRequest()
            } else {
                ToastUtil.showMessage("请授权获取手机信息后重试")
            }
        }
    }

    private fun initView() {
        initBarView()
        initQRView()
        refreshView.setOnClickListener {
            PaymentCodeStatisticUtil.commonClick("njfkm", "njfkmsx")
            stopTimeService()
            startTimeService()
            showUpdateSuccess()
            refreshQRCode()
        }
        paymentMethod.setOnClickListener {
            PaymentCodeStatisticUtil.commonClick("njfkm", "njfkmqh")
            if (mPaymentMethodModel != null) {
                showPaymentMethodDialog(mPaymentMethodModel!!)
            }
        }
        refresh.setOnClickListener {
            PaymentCodeStatisticUtil.commonClick("Mynj", "MynjSX")
            if (PaymentCodeCommonUtil.isNetConnected()) {
                initData()
            }
        }
        openButton.setOnClickListener {
            PaymentCodeStatisticUtil.commonClick("njfkm", "njfkmkq")
            PaymentCodeOpenUtil.startOpen(activity!!, openFaceCheckbox.isChecked) { isSuccess, msg ->
                if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this)) {
                    if (isSuccess) {
                        isPaymentSwitchOn = true
                        initData()
                    } else {
                        ToastUtil.showMessage(msg)
                    }
                }
            }
        }
        openFaceText.setOnClickListener {
            openFaceCheckbox.isChecked = !openFaceCheckbox.isChecked
        }

    }

    private fun initBarView() {
        val scaleW = PaymentCodeImageConstants.DISLAY_WIDTH.toFloat() / PaymentCodeImageConstants.VIEW_WIDTH.toFloat()
        val scaleH = PaymentCodeImageConstants.DISLAY_HEIGHT.toFloat() / PaymentCodeImageConstants.VIEW_HIGHT.toFloat()
        val width = (PaymentCodeImageConstants.VIEW_BARCODE_CONTENT_WIDTH * scaleW).toInt()
        val height = (PaymentCodeImageConstants.VIEW_BARCODE_CONTENT_HEIGHT * scaleH).toInt()
        val params = LinearLayout.LayoutParams(width, height)
        params.gravity = Gravity.CENTER
        paymentBarLayout.layoutParams = params
        mBarCodeWidth = (PaymentCodeImageConstants.VIEW_BARCODE_WIDTH * scaleW).toInt()
        mBarCodeHeight = (PaymentCodeImageConstants.VIEW_BARCODE_HEIGHT * scaleH).toInt()
        val imgParams = RelativeLayout.LayoutParams(mBarCodeWidth, mBarCodeHeight)
        imgParams.addRule(RelativeLayout.CENTER_IN_PARENT)
        barView.layoutParams = imgParams
    }

    private fun initQRView() {
        val scale = PaymentCodeImageConstants.DISLAY_WIDTH.toFloat() / PaymentCodeImageConstants.VIEW_WIDTH.toFloat()
        mQRCodeSize = (PaymentCodeImageConstants.VIEW_QRCODE_SIZE * scale).toInt()
        val params = RelativeLayout.LayoutParams(mQRCodeSize, mQRCodeSize)
        params.topMargin = (20 * scale).toInt()
        params.addRule(RelativeLayout.CENTER_HORIZONTAL)
        qrView.layoutParams = params
    }

    private fun initData() {
        if (PaymentCodeCommonUtil.isNetConnected()) {
            //查询开关状态
            sendQueryPaymentSwitch { isSuccess, isOn ->
                if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this)) {
                    if (isSuccess) {
                        isPaymentSwitchOn = isOn
                        if (!isOn) {
                            showOpenView()
                            sendQueryFacePayStatus { success, msg ->
                                if (success) {
                                    if ("10" != msg) {
                                        faceCheckLayout.visibility = View.VISIBLE
                                    } else {
                                        faceCheckLayout.visibility = View.GONE
                                    }
                                } else {
                                    ToastUtil.showMessage(msg)
                                }
                            }
                        } else {
                            sendPaymentCodeAuthIdRequest()
                            sendQueryPaymentMethod("1") {
                                if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this)) {
                                    mPaymentMethodModel = it
                                    if (!it.methodList.isEmpty()) {
                                        setPayChannelName(it.methodList[0])
                                    }
                                }
                            }
                        }
                    } else {
                        if (!isPaymentSwitchOn) {
                            showNoNetworkView()
                        }
                    }
                }
            }
        }
    }

    private fun showQRCodeByID(id: String, isDefault: Boolean = false) {
        var code = PaymentBitmapUtil.createPaymentCodeByID(id, mPaymentCodeModel?.paymentFactorSec)
        if (isDefault) {
            code = id
        }
        val oldBarBitmap = mBarCodeBitmap
        mBarCodeBitmap = PaymentBitmapUtil.createBarCodeBitmap(code, mBarCodeWidth, mBarCodeHeight)
        if (mBarCodeBitmap != null) {
            barView.setImageBitmap(mBarCodeBitmap)
        }
        if (oldBarBitmap != null && !oldBarBitmap.isRecycled) {
            oldBarBitmap.recycle()
        }

        val oldQRBitmap = mQRCodeBitmap
        mQRCodeBitmap = PaymentBitmapUtil.createQRCodeBitmap(code, mQRCodeSize, mQRCodeSize)
        if (mQRCodeBitmap != null) {
            qrView.setImageBitmap(mQRCodeBitmap)
        }
        if (oldQRBitmap != null && !oldQRBitmap.isRecycled) {
            oldQRBitmap.recycle()
        }
        showPaymentView()
        if (!hasUpdateShowTime) {
            hasUpdateShowTime = true
        }
    }

    private fun showNoNetworkView() {
        openLayout.visibility = View.GONE
        networkError.visibility = View.VISIBLE
        paymentCodeView.visibility = View.GONE
    }

    private fun showPaymentView() {
        openLayout.visibility = View.GONE
        networkError.visibility = View.GONE
        paymentCodeView.visibility = View.VISIBLE
    }

    private fun showOpenView() {
        openLayout.visibility = View.VISIBLE
        networkError.visibility = View.GONE
        paymentCodeView.visibility = View.GONE
    }

    private fun showUpdateSuccess() {
        changeUpdateTip(true)
        Handler().postDelayed({
            if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this@PaymentCodeMainFragment)) {
                changeUpdateTip(false)
            }
        }, 2000L)
    }

    private fun changeUpdateTip(isSuccess: Boolean) {
        if (isSuccess) {
            refreshView.visibility = View.GONE
            updateSuccess.visibility = View.VISIBLE
        } else {
            refreshView.visibility = View.VISIBLE
            updateSuccess.visibility = View.INVISIBLE
        }
    }

    private var timer: Timer? = null
    private var refreshTask: TimerTask? = null
    private var orderTask: TimerTask? = null
    private fun startTimeService() {
        if (timer == null) {
            timer = Timer()
        } else {
            return
        }
        if (refreshTask == null) {
            refreshTask = object : TimerTask() {

                override fun run() {
                    activity?.runOnUiThread {
                        if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this@PaymentCodeMainFragment)) {
                            refreshQRCode()
                        }
                    }
                }
            }
        }
        if (orderTask == null) {
            orderTask = object : TimerTask() {

                override fun run() {
                    activity?.runOnUiThread {
                        sendQueryOrderToPay {
                            if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this@PaymentCodeMainFragment)) {
                                activity?.runOnUiThread {
                                    getOrderTime = System.currentTimeMillis()
                                    when {
                                        "S" == it.status -> {
                                            isSdkPayed = false
                                            sendPaySuccessRequest(it.orderId)
                                        }
                                        "MYNJ" == it.channel && !TextUtils.isEmpty(it.token) -> {
                                            isSdkPayed = true
                                            toSNPaySdk(it.orderId, it.token)
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        timer!!.schedule(refreshTask, 60 * 1000, 60 * 1000)
        timer!!.schedule(
            orderTask,
            mPaymentCodeModel?.paymentPollingTime ?: 500,
            mPaymentCodeModel?.paymentPollingTime ?: 500
        )
    }

    private fun stopTimeService() {
        refreshTask?.cancel()
        refreshTask = null
        orderTask?.cancel()
        orderTask = null
        timer?.cancel()
        timer = null
    }

    private fun refreshQRCode() {
        // 在这个地方判断付款码是否有效 注意匹配服务器时间
        if (isPaymentSwitchOn && !TextUtils.isEmpty(mPaymentCodeModel?.paymentAuthId)) {
            showQRCodeByID(mPaymentCodeModel!!.paymentAuthId)
            showUpdateSuccess()
            if (PaymentDateUtil.isDateOneBigger(
                    mPaymentCodeModel!!.paymentExpirationTime,
                    PaymentCodeTimeSyncUtils.getPaymentCodeCurrentTime()
                )
            ) {
                // 如果授权码有效时间小于一天  暂时写死  提醒用户
                if (PaymentDateUtil.dateGap(
                        mPaymentCodeModel!!.paymentExpirationTime,
                        PaymentCodeTimeSyncUtils.getPaymentCodeCurrentTime()
                    ) < 1 * 24 * 60 * 60 * 1000 && !PaymentCodeCommonUtil.isNetConnected()
                ) {
                    CustomAlertDialog.showNoTitleRightBtn(
                        activity!!.fragmentManager,
                        "付款码即将失效，请及时联网更新",
                        "知道了",
                        null,
                        true
                    )
                }
            }
        } else {
            if (!PaymentCodeCommonUtil.isNetConnected()) {
                showNoNetworkView()
            }
        }
    }

    private fun updatePaymentMethod(position: Int) {
        if (mPaymentMethodModel != null) {
            paymentMethod.visibility = View.VISIBLE
            val tempBean = mPaymentMethodModel!!.methodList[position]
            mPaymentMethodModel!!.methodList.removeAt(position)
            mPaymentMethodModel!!.methodList.add(0, tempBean)
            setPayChannelName(tempBean)
        } else {
            paymentMethod.visibility = View.GONE
        }
    }

    private fun sendPaySuccessRequest(orderId: String) {
        ProgressViewDialog.getInstance().showProgressDialog(activity)
        ProgressViewDialog.getInstance().setCannotDissmis()
        sendQueryPaySuccess(orderId) { data ->
            if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this)) {
                ProgressViewDialog.getInstance().dismissProgressDialog()
                if ("1" == data?.getString("state")) {
                    //开启 channelCode 业务渠道 acqId 收单单号 acqType 收单类型　01：支付收单
                    val url = data.getString("url") + "?channelCode=01&acqId=" + orderId + "&acqType=01"
                    PaymentCodeGlobalInfo.h5CallBack?.gotoH5WebVIew(activity!!, url)
                } else {
                    showSuccessDialog()
                }
            }
        }
    }

    private fun sendPaymentCodeAuthIdRequest() {
        if (ActivityCompat.checkSelfPermission(
                activity!!,
                Manifest.permission.READ_PHONE_STATE
            ) == PackageManager.PERMISSION_GRANTED
        ) {
            sendQueryPaymentCode(DeviceInfoUtil.getPhoneIMEI(activity)) { isSuccess, model, msg ->
                if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this)) {
                    if (isSuccess && model != null) {
                        val oldPollingTime = mPaymentCodeModel?.paymentPollingTime ?: 0L
                        mPaymentCodeModel = model
                        if (mPaymentCodeModel?.paymentPollingTime != oldPollingTime) {
                            stopTimeService()
                            startTimeService()
                        }
                        refreshQRCode()
                    } else {
                        showNoNetworkView()
                        ToastUtil.showMessage(msg)
                    }
                }
            }
        } else {
            requestPermissions(arrayOf(Manifest.permission.READ_PHONE_STATE), REQUEST_PHONE)
        }
    }

    private fun toSNPaySdk(orderId: String, token: String) {
        LogUtils.i("PaymentCodeMainFragment","toSNPaySdk")
        PaymentPayH5Activity.listener = object : PaymentPayH5Activity.PayResultListener {
            override fun onResult(isSuccess: Boolean) {
                if (isSuccess) {
                    sendPaySuccessRequest(orderId)
                } else {
                    ToastUtil.showMessage("支付失败!")
                }
            }
        }
        val intent = Intent(activity, PaymentPayH5Activity::class.java)
        intent.putExtra("token", token)
        startActivity(intent)
    }

    private fun setPayChannelName(bean: PaymentMethodModel.PaymentMethodBean) {
        if (!TextUtils.isEmpty(bean.name)) {
            paymentMethod.visibility = View.VISIBLE
            Glide.with(this)
                .load(bean.iconUrl)
                .apply(RequestOptions().placeholder(R.drawable.payment_bank_default))
                .into(paymentMethodIcon)
            paymentMethodName.text = bean.name
            paymentMethodInfo.text = bean.info
        }
    }

    private fun showPaymentMethodDialog(model: PaymentMethodModel) {
        PaymentCodeMethodDialog.show(activity!!,
            model,
            AdapterView.OnItemClickListener { _, _, position, _ ->
                if (position != 0) {
                    if (PaymentCodeCommonUtil.isNetConnected()) {
                        ProgressViewDialog.getInstance().showProgressDialog(activity)
                        ProgressViewDialog.getInstance().setCannotDissmis()
                        sendSetPaymentMethod(mPaymentMethodModel!!.methodList[position]) { isSuccess, msg ->
                            if (!ActivityLifeCycleUtil.isFragmentDestory(activity, this)) {
                                ProgressViewDialog.getInstance().dismissProgressDialog()
                                if (isSuccess) {
                                    updatePaymentMethod(position)
                                } else {
                                    ToastUtil.showMessage(msg)
                                }
                            }
                        }
                    } else {
                        updatePaymentMethod(position)
                    }
                }
            })
    }

    private fun showSuccessDialog() {
        CustomAlertDialog.showNoTitleTwoBtn(
            activity?.fragmentManager,
            "支付成功，账单信息请至苏宁金融APP查看",
            "取消",
            null,
            "去下载",
            {
                PaymentCodeGlobalInfo.h5CallBack?.gotoH5WebVIew(activity!!, PaymentNetworkConfig.loadSnjrUrl)
            },
            true
        )
    }

}
