package com.suning.mobile.epa.paymentcode.main

import com.android.volley.Response
import com.suning.mobile.epa.NetworkKits.net.VolleyErrorHelper
import com.suning.mobile.epa.NetworkKits.net.VolleyRequestController
import com.suning.mobile.epa.NetworkKits.net.basic.EpaEncrypt
import com.suning.mobile.epa.exchangerandomnum.ExchangeRmdNumUtil
import com.suning.mobile.epa.kits.EpaKitsApplication
import com.suning.mobile.epa.kits.utils.DeviceInfoUtil
import com.suning.mobile.epa.kits.utils.LogUtils
import com.suning.mobile.epa.paymentcode.PaymentCodeGlobalInfo
import com.suning.mobile.epa.paymentcode.network.PaymentNetworkConfig
import com.suning.mobile.epa.paymentcode.network.PaymentNetworkRequest
import com.suning.mobile.epa.paymentcode.utils.PaymentCodeCommonUtil
import com.suning.mobile.epa.paymentcode.utils.PaymentCodeSPUtil
import com.suning.mobile.epa.paymentcode.utils.PaymentCodeTimeSyncUtils
import org.apache.http.NameValuePair
import org.apache.http.message.BasicNameValuePair
import org.json.JSONArray
import org.json.JSONObject
import java.net.URLEncoder

    private const val TAG = "PaymentCodeMainPresenter"

    fun sendQueryPaymentCode(imei: String, callBack : (Boolean, PaymentCodeModel?, String) -> Unit) {
        val list = ArrayList<NameValuePair>()
        list.add(BasicNameValuePair("service", "create_online_payqrcode"))
        var imeiTemp = imei
        if (PaymentCodeCommonUtil.isBlank(imeiTemp)) {
            imeiTemp = DeviceInfoUtil.getPhoneIMEI(EpaKitsApplication.getInstance())
        }
        val json = JSONObject()
        json.put("userNo", ExchangeRmdNumUtil.getUser().accountNo)
        json.put("equipmentNo", imeiTemp)
        json.put("channel", PaymentCodeGlobalInfo.channel)
        json.put("version", "3.0.0")
        val data = EpaEncrypt.pbeLocalEncrypt(json.toString())
        val url = PaymentNetworkConfig.ftis2TradeUrl + "PaymentCode/authorizationCodeAging.do?data=" + URLEncoder.encode(data, "UTF-8")
        val request = PaymentNetworkRequest(url,
                Response.Listener {
                    LogUtils.json(it.result.toString())
                    if (PaymentNetworkConfig.SUCCESS == it.responseCode) {
                        val model = PaymentCodeModel(it.result)
                        callBack(true, model, "")
                        PaymentCodeSPUtil.saveNetworkDeltaTime(System.currentTimeMillis() - PaymentCodeTimeSyncUtils.timeResponse * 1000)
                    } else {
                        callBack(false, null, it.responseMsg)
                    }
                },
                Response.ErrorListener {
                    callBack(false, null, VolleyErrorHelper.getMessage(it))
                })
        VolleyRequestController.getInstance().addToRequestQueueWithoutCache(request, TAG)
    }

    fun sendQueryPaymentSwitch(callBack : (Boolean, Boolean) -> Unit) {
        val url = PaymentNetworkConfig.ftis2TradeUrl + "payQrcode/queryPaymentSetting.do?service=userSettingPayment"
        val request = PaymentNetworkRequest(
                url,
                Response.Listener {
                    LogUtils.json(it.result.toString())
                    if (PaymentNetworkConfig.SUCCESS == it.responseCode) {
                        val state = it.result.optString("state")
                        callBack(true, "1" == state)
                    } else {
                        callBack(false, false)
                    }
                },
                Response.ErrorListener {
                    callBack(false, false)
                })
        VolleyRequestController.getInstance().addToRequestQueueWithoutCache(request, TAG)
    }

    fun sendQueryOrderToPay(callBack: (PaymentOrderToPayModel) -> Unit) {
        val json = JSONObject()
        json.put("accountNo", ExchangeRmdNumUtil.getUser().accountNo)
        json.put("equipmentNo", DeviceInfoUtil.getPhoneIMEI(EpaKitsApplication.getInstance()))
        json.put("buildVersion", "2")
        json.put("bundleID", EpaKitsApplication.getInstance().packageName)
        json.put("phoneSystem", "ANDROID")
        json.put("channel", PaymentCodeGlobalInfo.channel)
        json.put("terminalNo", DeviceInfoUtil.getDeviceId(EpaKitsApplication.getInstance()))
        val data = EpaEncrypt.pbeLocalEncrypt(json.toString())
        val url = PaymentNetworkConfig.ftis2TradeUrl + "payQrcode/queryNoPayOrder.do?data=" + URLEncoder.encode(data, "UTF-8")
        var time = System.currentTimeMillis().toString()
        var hasCode = time.hashCode()
        LogUtils.i(TAG,hasCode.toString()+":" + time)
        val request = PaymentNetworkRequest(url,
                Response.Listener {
//                    LogUtils.i(TAG,hasCode.toString() +":"+ System.currentTimeMillis().toString()+" responseCode:"+it.responseCode)
                    if (PaymentNetworkConfig.SUCCESS == it.responseCode) {
                        LogUtils.json(it.result.toString())
                        callBack(PaymentOrderToPayModel(it.result))
                    }
                },Response.ErrorListener {
                     var str =  VolleyErrorHelper.getMessage(it)
//                      LogUtils.i(TAG,hasCode.toString() +":"+ System.currentTimeMillis().toString()+"errorMsg:"+str)
                 }
                )
        VolleyRequestController.getInstance().addToRequestQueueWithoutCache(request, TAG)
    }

    fun sendQueryPaymentMethod(source: String, callBack: (PaymentMethodModel) -> Unit) {
        val url = PaymentNetworkConfig.ftis2TradeUrl + "paytype/queryUserPayTypes.do?sourceSign=$source"
        val request = PaymentNetworkRequest(
                url,
                Response.Listener { response ->
                    LogUtils.json(response.result.toString())
                    if (PaymentNetworkConfig.SUCCESS == response.responseCode) {
                        val model = PaymentMethodModel(response.result)
                        callBack(model)
                        LogUtils.d("queryUserPayTypes success")
                    } else {
                        LogUtils.d("queryUserPayTypes failed")
                    }
                },
                Response.ErrorListener {
                    LogUtils.d("queryUserPayTypes error")
                })
        VolleyRequestController.getInstance().addToRequestQueueWithoutCache(request, TAG)
    }

    fun sendSetPaymentMethod(bean: PaymentMethodModel.PaymentMethodBean, callBack: (Boolean, String) -> Unit) {
        val json = JSONObject()
        json.put("isAvailable", "00")
        json.put("isSimplepwd", "00")
        json.put("password", "")
        json.put("sourceSign", "1")
        json.put("payTypeInfo", JSONArray().put(bean.jsonObject))
        val data = EpaEncrypt.pbeLocalEncrypt(json.toString())
        val list = ArrayList<NameValuePair>()
        list.add(BasicNameValuePair("service", "defineUserPayTypes"))
        list.add(BasicNameValuePair("data", URLEncoder.encode(data, "UTF-8")))
        val url = PaymentNetworkConfig.ftis2TradeUrl + "paytype/defineUserPayTypes.do?data=" + URLEncoder.encode(data, "UTF-8")
        val request = PaymentNetworkRequest(
                url,
                Response.Listener {
                    LogUtils.json(it.result.toString())
                    if (PaymentNetworkConfig.SUCCESS == it.responseCode) {
                        callBack(true, "")
                    } else {
                        callBack(false, it.responseMsg)
                    }
                },
                Response.ErrorListener {
                    callBack(false, VolleyErrorHelper.getMessage(it))
                })
        VolleyRequestController.getInstance().addToRequestQueueWithoutCache(request, TAG)
    }

    fun sendQueryPaySuccess(orderId: String, callBack: (JSONObject?) -> Unit) {
        val json = JSONObject()
        json.put("merchantOrderId", orderId)
        val data = EpaEncrypt.pbeLocalEncrypt(json.toString())
        val list = ArrayList<NameValuePair>()
        list.add(BasicNameValuePair("service", "queryPaySuccess"))
        list.add(BasicNameValuePair("data", URLEncoder.encode(data, "UTF-8")))
        val url = PaymentNetworkConfig.ftisUrl + "paySuccess/queryPaySuccess.do?service=queryPaySuccess&data=" + URLEncoder.encode(data, "UTF-8")
        val request = PaymentNetworkRequest(url,
                Response.Listener {
                    LogUtils.json(it.result.toString())
                    if (PaymentNetworkConfig.SUCCESS == it.responseCode) {
                        callBack(it.result)
                    } else {
                        callBack(null)
                    }
                },
                Response.ErrorListener {
                    callBack(null)
                })
        VolleyRequestController.getInstance().addToRequestQueueWithoutCache(request, TAG)
    }

    fun sendQueryFacePayStatus(callBack: (Boolean, String) -> Unit){
        val url = PaymentNetworkConfig.ftis2TradeUrl + "payment/faceAccountQuery"
        val request = PaymentNetworkRequest(url,
                Response.Listener {
                    LogUtils.json(it.result.toString())
                    val code = it.result.optString("errorCode")
                    if (PaymentNetworkConfig.SUCCESS == code) {
                        val configureFileValue = it.result.optString("configureFileValue")
                        //10开启 20关闭
                        callBack(true, configureFileValue)
                    } else {
                        val msg = it.result.optString("desc")
                        callBack(false, msg)
                    }
                },
                Response.ErrorListener {
                    callBack(false, VolleyErrorHelper.getMessage(it))
                })
        VolleyRequestController.getInstance().addToRequestQueueWithoutCache(request, TAG)
    }
