package com.suning.mobile.epa.paymentcode.main

import android.text.TextUtils
import org.json.JSONObject

class PaymentMethodModel(jsonObject: JSONObject) {
    val methodList = ArrayList<PaymentMethodBean>()
    var isCacheData = false
    init {
        if (jsonObject.has("responseData")) {
            val data = jsonObject.getJSONObject("responseData")
            if (data.has("sortpayTypes")) {
                val array = data.getJSONArray("sortpayTypes")
                for (i in 0 until array.length()) {
                    methodList.add(PaymentMethodBean(array.getJSONObject(i)))
                }
            }
        }
    }

    class PaymentMethodBean(val jsonObject: JSONObject) {
        // 支付方式编码 rcrCode 支付方式编码
        // 快捷签约ID quickPayAuthId 快捷签约ID
        // 银行名称 bankName 银行名称
        // 银行卡类型 cardType 银行卡类型
        // 银行卡后四位 cardNoLater 银行卡后四位
        // 银行卡号（加密） cardNoEncrypt 银行卡号（加密）
        var name = ""
        var info = ""
        val iconUrl = jsonObject.optString("iconUrl")

        init {
            val rcrCode = jsonObject.optString("rcrCode")
            val bankName = jsonObject.optString("bankName")
            val cardNoLater = jsonObject.optString("cardNoLater")

            val EPP_BALANCE = "EPP_BALANCE"
            val BOF_BALANCE = "BOF_BALANCE"
            val BOF_CREDITPAYMENT = "BOF_CREDITPAYMENT"
            val EPP_CREDITPAYMENT = "EPP_CREDITPAYMENT"
            val DEBIT_QUICKPAYMENT = "DEBIT_QUICKPAYMENT"
            val CREDIT_QUICKPAYMENT = "CREDIT_QUICKPAYMENT"

            if (!TextUtils.isEmpty(rcrCode)){
                when {
                    EPP_BALANCE == rcrCode -> name = "账户余额"
                    BOF_BALANCE == rcrCode -> name = "零钱宝"
                    BOF_CREDITPAYMENT == rcrCode -> name = "零钱贷"
                    EPP_CREDITPAYMENT == rcrCode -> name = "任性付"
                    DEBIT_QUICKPAYMENT == rcrCode -> {
                        name = bankName + "储蓄卡"
                        info = "($cardNoLater)"
                    }
                    CREDIT_QUICKPAYMENT == rcrCode -> {
                        name = bankName + "信用卡"
                        info = "($cardNoLater)"
                    }
                }
            }
        }


    }
}