package com.suning.mobile.epa.paymentcode.network

import android.text.TextUtils
import com.android.volley.NetworkResponse
import com.android.volley.Response
import com.suning.mobile.epa.NetworkKits.net.NetworkBeanRequest
import com.suning.mobile.epa.NetworkKits.net.basic.NetworkBean
import com.suning.mobile.epa.NetworkKits.net.other.TimeUtil
import com.suning.mobile.epa.kits.utils.LogUtils
import com.suning.mobile.epa.paymentcode.PaymentCodeGlobalInfo
import com.suning.mobile.epa.paymentcode.utils.PaymentCodeTimeSyncUtils

class PaymentNetworkRequest(url: String, listener: Response.Listener<NetworkBean>?, errorListener: Response.ErrorListener?) : NetworkBeanRequest(url, listener, errorListener) {

    override fun parseNetworkResponse(response: NetworkResponse?): Response<NetworkBean> {
        var cacheDate = System.currentTimeMillis() / 1000;
        if (response?.headers != null) {
            cacheDate = getDateFromCache(response.headers.getValue("Date"))
        }
        PaymentCodeTimeSyncUtils.setTimeFromResponse(cacheDate)
        return super.parseNetworkResponse(response)
    }

    private fun getDateFromCache(date : String) : Long{
        var cacheDate = System.currentTimeMillis()
        if (!TextUtils.isEmpty(date)) {
            val serviceDate = TimeUtil.formatGMTDate(date)
            if (serviceDate != null) {
                cacheDate = serviceDate.time
            }
        }
        return cacheDate / 1000
    }

    override fun getHeaders(): MutableMap<String, String> {
        val map = super.getHeaders()
        val typeKey = "terminalType"
        val versionKey = "eppVersion"
        map[typeKey] = PaymentCodeGlobalInfo.source.result
        map[versionKey] = PaymentCodeGlobalInfo.version

        return map
    }

    override fun deliverResponse(response: NetworkBean?) {
        if ("5015" == response?.responseCode){
//            LogUtils.i("PaymentNetworkRequest","5015 url:"+originUrl)
            PaymentCodeGlobalInfo.errorCallBack?.showErrorView()
        }
        super.deliverResponse(response)
    }
}