package com.suning.mobile.epa.paymentcode.utils

import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Matrix
import android.graphics.Paint
import android.text.TextUtils
import com.google.zxing.BarcodeFormat
import com.google.zxing.EncodeHintType
import com.google.zxing.MultiFormatWriter
import com.google.zxing.qrcode.QRCodeWriter
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel
import com.suning.mobile.epa.kits.EpaKitsApplication
import com.suning.mobile.epa.kits.utils.ResUtil

import com.suning.mobile.epa.paymentcode.encode.PayCodeThreeJNI

object PaymentBitmapUtil {
    private val TAG = "PaymentBitmapUtil"

    /** 码默认背景颜色  */
    private var CODE_BG_COLOR = 0xFFFFFFFF.toInt()
    /** 二维码的背景颜色025eae  */
    private val QRCODE_COLOR = 0xF8F8F8
    /** 码颜色（通常我们看到的是黑色）  */
    private val BLACK = 0xFF000000.toInt()

    private fun isUTF8(msg: CharSequence): Boolean {
        for (c in msg) {
            if (c.toInt() > 0xFF) {
                return true
            }
        }
        return false
    }

    fun createPaymentCodeByID(id: String, key : String?): String {
        if (TextUtils.isEmpty(id) || TextUtils.isEmpty(key)) {
            return ""
        }
        val generatorCode = PayCodeThreeJNI()
        return "8" + generatorCode.createpaycode(
                PaymentCodeTimeSyncUtils.getPaymentCodeCurrentTime(),
                id,
                key!!)
    }

    fun createBarCodeBitmap(data: String, width: Int, height: Int): Bitmap? {
        if (TextUtils.isEmpty(data)) {
            return null
        }
        val hints: MutableMap<EncodeHintType, Any> = HashMap()
        if (isUTF8(data)) {
            hints.put(EncodeHintType.CHARACTER_SET, "UTF-8")
            hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H)
        }
        val bit = MultiFormatWriter().encode(data,
                BarcodeFormat.CODE_128,
                width,
                height,
                hints)
        val pixels = IntArray(width * height)
        for (y in 0 until height) {
            val offset = y * width
            for (x in 0 until width) {
                pixels[offset + x] = if (bit.get(x, y)) BLACK else CODE_BG_COLOR
            }
        }
        val bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
        bitmap.setPixels(pixels, 0, width, 0, 0, width, height)
        return bitmap
    }

    fun createQRCodeBitmap(data: String, width: Int, height: Int): Bitmap? {
        if (TextUtils.isEmpty(data)) {
            return null
        }
        val hints: MutableMap<EncodeHintType, Any> = HashMap()
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8")
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H)
        hints.put(EncodeHintType.MARGIN, 0)

        val bit = QRCodeWriter().encode(data,
                BarcodeFormat.QR_CODE,
                width,
                height,
                hints)
        val pixels = IntArray(width * height)
        for (y in 0 until height) {
            val offset = y * width
            for (x in 0 until width) {
                pixels[offset + x] = if (bit.get(x, y)) BLACK else CODE_BG_COLOR
            }
        }
        val bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
        bitmap.setPixels(pixels, 0, width, 0, 0, width, height)
        return bitmap
    }
}
