package com.suning.mobile.epa.paymentcode.utils


import android.text.TextUtils
import com.suning.mobile.epa.kits.utils.LogUtils

import java.io.UnsupportedEncodingException
import java.net.URLDecoder

/**
 * 用于处理H5的url相关操作
 * Created by 14043047 on 2015/11/2.
 */
object UrlUtils {
    private const val URL_TARGETURL = "targetUrl="
    private const val URL_SERVICEURL = "service="
    private const val SUNING_DOMAIN = ".suning.com"
    private const val CNSUNING_DOMAIN = ".cnsuning.com"

    private const val PERIOD = '.'
    private const val SLASH = "/"
    private const val COLON = ":"

    fun getTargetUrl(url: String): String {
        var targetUrl = url
        //保留原地址
        val originalUrl = targetUrl
        //获取原url里的targetUrl
        try {
            //如果含有service=，截取service=相关
            if (targetUrl.contains(URL_SERVICEURL)) {
                targetUrl = targetUrl.substring(targetUrl.lastIndexOf(URL_SERVICEURL) + URL_SERVICEURL.length)
                if (targetUrl.contains("&")) {
                    targetUrl = targetUrl.substring(0, targetUrl.indexOf("&"))
                }
                targetUrl = URLDecoder.decode(targetUrl, "UTF-8")

                //如果还含有targetUrl=，截取targetUrl=相关
                if (targetUrl.contains(URL_TARGETURL)) {
                    targetUrl = targetUrl.substring(targetUrl.lastIndexOf(URL_TARGETURL) + URL_TARGETURL.length)
                    if (targetUrl.contains("&")) {
                        targetUrl = targetUrl.substring(0, targetUrl.indexOf("&"))
                    }
                    targetUrl = URLDecoder.decode(targetUrl, "UTF-8")
                }
            } else if (targetUrl.contains(URL_TARGETURL)) {//如果只有有targetUrl=，截取targetUrl=相关
                targetUrl = targetUrl.substring(targetUrl.lastIndexOf(URL_TARGETURL) + URL_TARGETURL.length)
                if (targetUrl.contains("&")) {
                    targetUrl = targetUrl.substring(0, targetUrl.indexOf("&"))
                }
                targetUrl = URLDecoder.decode(targetUrl, "UTF-8")
            }

            //判断是否为非suning域名，如果是返回原地址拼接&gateway=true
            val domain = getHostDomain(targetUrl)
            if (!domain.contains(SUNING_DOMAIN) && !domain.contains(CNSUNING_DOMAIN)) {
                targetUrl = "$originalUrl&gateway=true"
            }
        } catch (e: UnsupportedEncodingException) {
            LogUtils.d("WebViewUtils", "getTargetUrl decode exception")
        }
        return targetUrl
    }


    fun getHostDomain(loadUrl: String): String {
        val host = obtainHost(loadUrl) // 获取cookie路径
        return if (isIpAddress(host)) {
            host
        } else {
            getDomain(host)
        }
    }

    private fun obtainHost(loadUrl: String): String{
        val host: String?
        val url = loadUrl.split(SLASH.toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
        if (loadUrl.contains(COLON) && url.size > 2) {
            if (url[2].contains(COLON)) {
                val urlTmp = url[2].split(COLON.toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                url[2] = urlTmp[0]
            }
            host = url[2]
        } else {
            host = url[0]
        }
        return host
    }

    private fun isIpAddress(host: String): Boolean {
        if (!TextUtils.isEmpty(host)) {
            // 定义正则表达式
            val regex = ("^(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|[1-9])\\."
                    + "(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)\\."
                    + "(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)\\."
                    + "(1\\d{2}|2[0-4]\\d|25[0-5]|[1-9]\\d|\\d)$")
            // 判断ip地址是否与正则表达式匹配
            return host.matches(regex.toRegex())
        }
        // 返回判断信息
        return false
    }

    /**
     * @Description 通过所请求的host获取domain
     * @param host 不可以"/"结尾
     * @return
     * @Author dlobo
     * @Date 2013-11-29
     */
    private fun getDomain(host: String): String {
        var startIndex = 0
        var nextIndex = host.indexOf(PERIOD)
        val lastIndex = host.lastIndexOf(PERIOD)
        while (nextIndex < lastIndex) {
            startIndex = nextIndex + 1
            nextIndex = host.indexOf(PERIOD, startIndex)
        }
        return if (startIndex > 0) {
            "." + host.substring(startIndex)
        } else {
            host
        }
    }
}
