package com.suning.mobile.epa.paymentcode.view

import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.graphics.drawable.ColorDrawable
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.suning.epafusion.R
import com.suning.mobile.epa.NetworkKits.net.VolleyRequestController
import com.suning.mobile.epa.kits.utils.ToastUtil
import com.suning.mobile.epa.paymentcode.PaymentCodeGlobalInfo
import com.suning.mobile.epa.paymentcode.main.PaymentMethodModel

/**
 * RxdCommonPopupWindow.kt
 * <功能> 通用的底部弹框
 * <详细>
 *
 * @author yanss 16072015
 * @createdTime 2018/3/5 0005 11:56:17
 */
object PaymentCodeMethodDialog{

    private var dialog : Dialog? = null
    fun show(context : Context,
             model: PaymentMethodModel,
             listener: AdapterView.OnItemClickListener) {
        val layoutInflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        val view = layoutInflater.inflate(R.layout.popup_window_payment_code_method, null)
        initView(context, view, model, listener)
        dialog = Dialog(context, R.style.PaymentDialogFullscreen)
        dialog?.setContentView(view)
        dialog?.show()
    }



    private fun initView(context : Context,
                         view: View,
                         model: PaymentMethodModel,
                         listener: AdapterView.OnItemClickListener) {
        val close = view.findViewById<View>(R.id.btn_back)
        close.setOnClickListener {
            dialog?.dismiss()
        }

        val listView = view.findViewById<View>(R.id.pay_list) as ListView
        listView.adapter = PaymentMethodListAdapter(context, model)
        listView.setOnItemClickListener { parent, v, position, id ->
            listener.onItemClick(parent, v, position, id)
            dialog?.dismiss()
        }
    }

    class PaymentMethodListAdapter(val context: Context, private val model : PaymentMethodModel) : BaseAdapter() {

        override fun getView(position: Int, convertView: View?, parent: ViewGroup?): View {
            val holder: ViewHolder
            var view = convertView
            if (view == null) {
                val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
                view = inflater.inflate(R.layout.item_payment_code_method, null)
                holder = ViewHolder()
                view.tag = holder
                holder.payImg = view.findViewById(R.id.pay_img) as ImageView
                holder.payName = view.findViewById(R.id.pay_name) as TextView
                holder.payInfo = view.findViewById(R.id.pay_info) as TextView
                holder.payChecked = view.findViewById(R.id.pay_checked) as ImageView
            } else {
                holder = view.tag as ViewHolder
            }
            val bean = model.methodList[position]
            Glide.with(context)
                    .load(bean.iconUrl)
                    .apply(RequestOptions().placeholder(R.drawable.payment_bank_default))
                    .into(holder.payImg)
            holder.payName?.text = bean.name
            holder.payInfo?.text = bean.info
            if (position == 0) {
                holder.payChecked!!.visibility = View.VISIBLE
            } else {
                holder.payChecked!!.visibility = View.GONE
            }
            return view!!
        }

        override fun getItem(position: Int): Any {
            return model.methodList[position]

        }

        override fun getItemId(position: Int): Long {
            return position.toLong()
        }

        override fun getCount(): Int {
            return model.methodList.count()
        }
    }

    private class ViewHolder {
        var payName: TextView? = null
        var payInfo: TextView? = null
        var payImg: ImageView? = null
        var payChecked: ImageView? = null
    }
}
